// --- Initialization ---

//run("Close All");
run("Options...", "iterations=3 count=1 edm=Overwrite");
setBatchMode(false);

sFileDir=getDirectory("Choose a Directory");
File.makeDirectory(sFileDir+"3D alignment");
sFileDirFolder=sFileDir+"3D alignment/"
fName=getTitle();
File.makeDirectory(sFileDirFolder+fName);
sFileDirOut=sFileDirFolder+fName+"/";


run("3D Manager");
selectWindow(fName);
run("Duplicate...", "title=Original duplicate");
run("Split Channels");
imgNameC1="C1-Original";
imgNameC2="C2-Original";


// --- create mask for 3d analysis---
selectWindow(imgNameC1);
Stack.getDimensions(width, height, channels, slices, frames);
run("Duplicate...", "title=CopyC1 duplicate");
run("Median...", "radius=2 stack");
run("Convert to Mask", "method=Moments background=Dark");
run("Dilate", "stack");
run("Fill Holes", "stack");
run("Analyze Particles...", "size=250-Infinity pixel show=Masks stack");
rename("Mask");
selectWindow("CopyC1");
close();
selectWindow("Mask");

// --- 3D segmentation ---

run("3D Manager Options", "volume surface compactness fit_ellipse integrated_density mean_grey_value std_dev_grey_value minimum_grey_value maximum_grey_value centroid_(pix) centroid_(unit) distance_to_surface centre_of_mass_(pix) centre_of_mass_(unit) bounding_box radial_distance closest exclude_objects_on_edges_xy exclude_objects_on_edges_z distance_between_centers=10 distance_max_contact=1.80");
Ext.Manager3D_Segment(128, 255);
rename("3dMask");
selectWindow("Mask");
close();
Stack.getStatistics(voxelCount, mean, min, max, stdDev);

// --- 3D Alignment ---

for (obj=1; obj<=max;obj++){
	selectWindow("3dMask");
	run("Duplicate...", "title=3dMask-copy duplicate");
	setThreshold(obj, obj);
	run("Convert to Mask", "method=Default background=Dark");
	run("Dilate (3D)", "iso=255");
	
	imageCalculator("AND create stack", imgNameC1,"3dMask-copy");
	run("Properties...", "channels=1 slices=1 frames="+slices+" unit=microns pixel_width=0.1002085 pixel_height=0.1002085 voxel_depth="+0.3776549/2);
	imageCalculator("AND create stack", imgNameC2,"3dMask-copy");
	run("Properties...", "channels=1 slices=1 frames="+slices+" unit=microns pixel_width=0.1002085 pixel_height=0.1002085 voxel_depth="+0.3776549/2);
	run("Merge Channels...", "c1=[Result of C2-Original] c2=[Result of C1-Original] create");
	rename("Object "+obj+"_NotAligned");
	
	run("Correct 3D drift", "channel=2 only=0 lowest=1 highest=1");
	wait(200);
	selectWindow("registered time points");
	run("Properties...", "channels=2 slices="+slices+" frames=1 unit=microns pixel_width=0.1002085 pixel_height=0.1002085 voxel_depth="+0.3776549/2);
	selectWindow("registered time points");
	saveAs("Tiff",sFileDirOut+"Object "+obj);
	wait(1000);
	selectWindow("Object "+obj+".tif");
	close();
	selectWindow("Object "+obj+"_NotAligned");
	close();	
	selectWindow("3dMask-copy");
	close();
	print("Finished: Object "+obj+" out of "+max);	
}
selectWindow("3dMask");
close();
selectWindow(imgNameC1);
close();
selectWindow(imgNameC2);
close();
run("Options...", "iterations=1 count=1 edm=Overwrite");
print("Analysis complete");
