////////////////////////////////
////////   3D Analysis  ////////
////////   by A. Dias   ////////
////////////////////////////////

// --- version 1.00 (25/07/2016) ---
// - This macro was designed to measure geometrical and shape descriptors for 3d objects.
// - Requirements: [3D ImageJ Suite] and [ImageScience] websites toogled as update sites.

// --- Initialization ---
run("Close All");
run("Options...", "iterations=2 count=1 edm=Overwrite");
setBatchMode(true);

showMessage("Select input folder");
sFileDir=getDirectory("Choose a Directory");
list = getFileList(sFileDir);

// --- creating a new directory ---

File.makeDirectory(sFileDir+"3D Analysis");
sFileDirOut=sFileDir+"3D Analysis"+"/";// --- For Windows, change "/" with "\\"

// --- Prompt for and split background image ---

for (i = 0; i < list.length; i++) {
	if(endsWith(list[i],"tif")) {
		open(list[i]);
		fTitle=getTitle();
		fName=File.nameWithoutExtension;

//		run("Duplicate...", "title="+fName+" duplicate");
		run("Split Channels");
		imgC1="C1-"+fTitle;
		imgC2="C2-"+fTitle;

// --- Pre-processing the image ---
		selectWindow(imgC1);
		close();
		selectWindow(imgC2);
		run("Median...", "radius=1 stack");
		run("Convert to Mask", "method=Moments background=Dark calculate");
		run("Close-", "stack");
		run("3D Fill Holes");

// --- Measure 	parameters ---
		run("3D Geometrical Measure");
		selectWindow("Results");
		save(sFileDirOut+fName+"_GeoMeas.txt");
		run("Clear Results");
		
		selectWindow("C2-Object::segmented");
		Stack.getStatistics(voxelCount, mean, min, max, stdDev) 
		setMinAndMax(0, max);
		run("8-bit");
		saveAs("Tiff", sFileDirOut+fName+"_3dMask");
		wait(1000);
		selectWindow(fName+"_3dMask.tif");
		close();

		selectWindow(imgC2);
		run("3D Shape Measure");
		selectWindow("Results");
		save(sFileDirOut+fName+"_ShapeMeas.txt");
		run("Clear Results");
		selectWindow("C2-Object::segmented");
		close();
		print("File analyzed: "+list[i]+" ("+(i+1)/(list.length)*100+" %)");

		selectWindow(imgC2);
		close();
	}
}
run("Options...", "iterations=1 count=1 edm=Overwrite");	